<?php
/**
 * Authorize.Net API wrapper.
 *
 * @since 1.7.0
 *
 * @package WC_Authorize_Net_DPM
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * API Wrapper for Authorize.Net.
 */
class WC_Authorize_Net_API {

	const SANDBOX_URL = 'https://apitest.authorize.net/xml/v1/request.api';
	const PRODUCTION_URL = 'https://api.authorize.net/xml/v1/request.api';

	const PRODUCTION_SOLUTION_ID = 'A1000065';
	const SANDBOX_SOLUTION_ID = 'AAA100302';

	/**
	 * Makes a request to Authorize.Net API.
	 *
	 * @since 1.7.0
	 *
	 * @param string $type Request type.
	 * @param array  $args Request args.
	 *
	 * @return WP_Error|array
	 */
	protected static function _request( $type, $args = array() ) {
		/**
		 * Authorize.Net has JSON support that still in beta which requires
		 * merchants to contact them to activate. At this point separating the
		 * array body generator into its own method to make it easy when JSON
		 * is widely supported by Authorize.net.
		 */
		$array_body = self::_get_array_body_request( $type, $args );
		$xml_body   = self::_get_xml_body_request( $array_body );

		$request = array(
			'method'  => 'POST',
			'timeout' => 30,
			'body'    => $xml_body,
			'headers' => array(
				'Content-Type' => 'text/xml',
			),
		);

		wc_authorize_net_dpm()->log( sprintf( '%s - Request: %s', __METHOD__, print_r( $request, true ) ) );

		$resp = wp_remote_request( self::get_api_url(), $request );
		if ( is_wp_error( $resp ) ) {
			wc_authorize_net_dpm()->log( sprintf( '%s - Error %s', __METHOD__, $resp->get_error_message() ) );
			return $resp;
		}

		return self::_get_parsed_response( $type, $resp );
	}

	/**
	 * Get body request from a given type request.
	 *
	 * Type will be the wrapper of XML request. For example 'createTransactionRequest'
	 * for capture, refund, and void. This also adds common elements such as
	 * merchantAuthentication. Additional args passed to self::_request will be
	 * appended after that.
	 *
	 * @since 1.7.0
	 *
	 * @param string $type Request type. Root element of XML request.
	 * @param array  $args Request args.
	 *
	 * @return array Base request.
	 */
	protected static function _get_array_body_request( $type, $args = array() ) {
		$settings = wc_authorize_net_dpm()->get_settings();

		$body = array(
			'name'       => $type,
			'attributes' => array( 'xmlns' => 'AnetApi/xml/v1/schema/AnetApiSchema.xsd' ),
			array(
				'name' => 'merchantAuthentication',
				array(
					'name'  => 'name',
					'value' => $settings['api_login'],
				),
				array(
					'name'  => 'transactionKey',
					'value' => $settings['transaction_key'],
				),
			),
		);

		if ( ! empty( $args ) ) {
			$body[] = $args;
		}

		return apply_filters( 'wc_authorize_net_dpm_api_array_body_request', $body );
	}

	/**
	 * Get XML request to be sent to Authorize.Net API.
	 *
	 * Props to viper007bond for this.
	 *
	 * @see http://www.viper007bond.com/2011/06/29/easily-create-xml-in-php-using-a-data-array/
	 *
	 * @since 1.7.0
	 *
	 * @param array $request Request.
	 *
	 * @return string XML request.
	 */
	protected static function _get_xml_body_request( array $request ) {
		$doc   = new DOMDocument();
		$child = self::_generate_xml_element( $doc, $request );
		if ( $child ) {
			$doc->appendChild( $child );
		}

		return $doc->saveXML();
	}

	/**
	 * Generates XML element from a given array data.
	 *
	 * @since 1.7.0
	 *
	 * @see http://www.viper007bond.com/2011/06/29/easily-create-xml-in-php-using-a-data-array/
	 *
	 * @param DOMDocument $dom  DOM.
	 * @param array       $data XML represented in array.
	 *
	 * @return bool|DOMElement Returns false or DOMElement.
	 */
	protected static function _generate_xml_element( $dom, $data ) {
		if ( empty( $data['name'] ) ) {
			return false;
		}

		// Create the element.
		$element_value = ( ! empty( $data['value'] ) ) ? $data['value'] : null;
		$element       = $dom->createElement( $data['name'], $element_value );

		// Add any attributes.
		if ( ! empty( $data['attributes'] ) && is_array( $data['attributes'] ) ) {
			foreach ( $data['attributes'] as $attribute_key => $attribute_value ) {
				$element->setAttribute( $attribute_key, $attribute_value );
			}
		}

		// Any other items in the data array should be child elements.
		foreach ( $data as $data_key => $child_data ) {
			if ( ! is_numeric( $data_key ) ) {
				continue;
			}

			$child = self::_generate_xml_element( $dom, $child_data );
			if ( $child ) {
				$element->appendChild( $child );
			}
		}

		return $element;
	}

	/**
	 * Constructs SimpleXMLElement from source (XML string).
	 *
	 * @since 1.7.0
	 *
	 * @param string $source  The string containing XML.
	 * @param int    $options Bitwise OR of the libxml  option constants.
	 *
	 * @return bool|SimpleXMLElement Returns false or SimpleXMLElement.
	 */
	protected static function _safe_load_xml( $source, $options ) {
		$prev = null;

		// See http://php.net/manual/en/function.libxml-disable-entity-loader.php.
		if ( function_exists( 'libxml_disable_entity_loader' ) ) {
			$prev = libxml_disable_entity_loader( true );
		}

		$dom    = new DOMDocument();
		$return = $dom->loadXML( $source, $options );

		// Set back the previous value.
		if ( ! is_null( $prev ) ) {
			libxml_disable_entity_loader( $prev );
		}

		if ( ! $return ) {
			return false;
		}

		if ( isset( $dom->doctype ) ) {
			return false;
		}

		return simplexml_import_dom( $dom );
	}

	/**
	 * Get array representation from remote response.
	 *
	 * @since 1.7.0
	 *
	 * @param string $type Request type. Root element of XML request.
	 * @param array  $resp Response from wp_remote_request.
	 *
	 * @return WP_Error|array Returns WP_Error or array representation of response.
	 */
	protected static function _get_parsed_response( $type, $resp ) {
		$body = wp_remote_retrieve_body( $resp );

		wc_authorize_net_dpm()->log( sprintf( '%s - Response body: %s', __METHOD__, $body ) );

		$xml_obj = self::_safe_load_xml( $body, LIBXML_NOCDATA | LIBXML_NOWARNING );
		if ( ! $xml_obj ) {
			return new WP_Error( 'xml_load_failed', __( 'Failed to load XML from response body', 'woocommerce-gateway-authorize-net-dpm' ) );
		}

		// Normalize XML object to array based on reques type.
		switch ( $type ) {
			case 'createTransactionRequest':
			case 'getTransactionDetailsRequest':
			case 'authenticateTestRequest':
				return self::_get_parsed_xml_object_response( $xml_obj );
		}

		return new WP_Error( 'unhandled_type', __( 'Unhandled response type', 'woocommerce-gateway-authorize-net-dpm' ) );
	}

	/**
	 * Get parsed array of XML object response.
	 *
	 * @since 1.7.0
	 *
	 * @param SimpleXMLElement $xml_obj XML Element of response body.
	 *
	 * @return WP_Error|array Returns WP_Error or array representation of response.
	 */
	protected static function _get_parsed_xml_object_response( $xml_obj ) {
		$resp = self::_xml_object_to_array( $xml_obj );
		if ( ! $resp ) {
			return new WP_Error( 'failed_xml_to_array', __( 'Unexpected result of xml object to array conversion', 'woocommerce-gateway-authorize-net-dpm' ) );
		}

		if ( empty( $resp['messages']['resultCode'] ) ) {
			return new WP_Error( 'missing_result_code', __( 'Missing result code in the response', 'woocommerce-gateway-authorize-net-dpm' ) );
		}

		if ( 'Ok' !== trim( $resp['messages']['resultCode'] ) ) {
			return new WP_Error( 'result_code_not_ok', self::_get_error_message_from_parsed_response( $resp ) );
		}

		return $resp;
	}

	/**
	 * Get error message from parsed response.
	 *
	 * @since 1.7.0
	 *
	 * @param array $resp Parsed response in array.
	 *
	 * @return string Error message.
	 */
	protected static function _get_error_message_from_parsed_response( $resp ) {
		$messages = array();

		$messages[] = ! empty( $resp['messages']['message']['text'] ) ? $resp['messages']['message']['text'] : __( 'Unexpected API response', 'woocommerce-gateway-authorize-net-dpm' );

		if ( ! empty( $resp['transactionResponse']['errors']['error'] ) ) {
			$errors = $resp['transactionResponse']['errors']['error'];

			// A single error returned.
			if ( ! empty( $errors['errorCode'] ) && ! empty( $errors['errorText'] ) ) {
				$errors = array( $errors );
			}

			foreach ( $errors as $i => $error ) {
				if ( ! empty( $error['errorCode'] ) && ! empty( $error['errorText'] ) ) {
					$messages[] = $i + 1 . ') ' . sprintf( __( 'Error code %1$s: %2$s', 'woocommerce-gateway-authorize-net-dpm' ), $error['errorCode'], $error['errorText'] );
				}
			}
		}

		return implode( ' ', $messages );
	}

	/**
	 * Get array representation of XML object. Note that this approach is a bit
	 * hacky.
	 *
	 * @since 1.7.0
	 *
	 * @param SimpleXMLElement $xml_obj XML object.
	 *
	 * @return array Array representation of XML object.
	 */
	protected static function _xml_object_to_array( $xml_obj ) {
		return json_decode( wp_json_encode( (array) $xml_obj ), true );
	}

	/**
	 * Get Authorize.Net API URL.
	 *
	 * @since 1.7.0
	 *
	 * @return string API URL
	 */
	public static function get_api_url() {
		$settings = wc_authorize_net_dpm()->get_settings();

		if ( 'yes' === $settings['testmode'] ) {
			return self::SANDBOX_URL;
		}

		return self::PRODUCTION_URL;
	}

	/**
	 * Test authenticatation.
	 *
	 * @since 1.7.0
	 *
	 * @return WP_Error|array See self::_request return value
	 */
	public static function authenticate() {
		return self::_request( 'authenticateTestRequest' );
	}

	/**
	 * Capture funds for a transaction in given order_id that was previously
	 * authorized.
	 *
	 * @since 1.7.0
	 *
	 * @param int|WC_Order $order Order ID or order object.
	 *
	 * @return WP_Error|array See self::_request return valuel
	 */
	public static function capture( $order ) {
		$order          = wc_get_order( $order );
		$transaction_id = wc_authorize_net_dpm_order_prop( $order, 'transaction_id' );

		if ( empty( $transaction_id ) ) {
			return new WP_Error( 'missing_transaction_id', __( 'Transaction ID is missing from the order', 'woocommerce-gateway-authorize-net-dpm' ) );
		}

		$settings = wc_authorize_net_dpm()->get_settings();

		$request = array(
			'name' => 'transactionRequest',
			array(
				'name'  => 'transactionType',
				'value' => 'priorAuthCaptureTransaction',
			),
			array(
				'name'  => 'amount',
				'value' => $order->get_total(),
			),
			array(
				'name' => 'solution',
				array(
					'name'  => 'id',
					'value' => 'yes' === $settings['testmode'] ? self::SANDBOX_SOLUTION_ID : self::PRODUCTION_SOLUTION_ID,
				),
			),
			array(
				'name'  => 'refTransId',
				'value' => wc_authorize_net_dpm_order_prop( $order, 'transaction_id' ),
			),
			array(
				'name' => 'order',
				array(
					'name'  => 'invoiceNumber',
					'value' => $order->get_order_number(),
				),
			),
		);

		return self::_request( 'createTransactionRequest', $request );
	}

	/**
	 * Refund a transaction in given order_id that was successfully settled.
	 *
	 * @since 1.7.0
	 *
	 * @param int|WC_Order $order  Order ID or order object.
	 * @param float        $amount Amount to refund.
	 * @param string       $note   Reson to refund. There's no field for reason
	 *                             in the API. Keeping it here in case it
	 *                             changes in the future.
	 *
	 * @return WP_Error|array See self::_request return value.
	 */
	public static function refund( $order, $amount, $note ) {
		$order          = wc_get_order( $order );
		$transaction_id = wc_authorize_net_dpm_order_prop( $order, 'transaction_id' );

		if ( empty( $transaction_id ) ) {
			return new WP_Error( 'missing_transaction_id', __( 'Transaction ID is missing from the order', 'woocommerce-gateway-authorize-net-dpm' ) );
		}

		$details = self::get_transaction_details( $order );
		if ( is_wp_error( $details ) ) {
			return new WP_Error( 'failed_get_transaction_details', sprintf( __( 'Failed to retrieve transaction details for order #%1$s: %2$s', 'woocommerce-gateway-authorize-net-dpm' ), wc_authorize_net_dpm_order_prop( $order, 'id' ), $details->get_error_message() ) );
		}

		if ( empty( $details['transaction']['payment']['creditCard']['cardNumber'] ) ) {
			return new WP_Error( 'missing_card_number', __( 'Missing the last four digits card number in transaction details', 'woocommerce-gateway-authorize-net-dpm' ) );
		}

		$settings = wc_authorize_net_dpm()->get_settings();

		$last_four_digits = substr( $details['transaction']['payment']['creditCard']['cardNumber'], -4 );

		$request = array(
			'name' => 'transactionRequest',
			array(
				'name'  => 'transactionType',
				'value' => 'refundTransaction',
			),
			array(
				'name'  => 'amount',
				'value' => wc_float_to_string( $amount ),
			),
			array(
				'name' => 'payment',
				array(
					'name' => 'creditCard',
					array(
						'name'  => 'cardNumber',
						'value' => $last_four_digits,
					),
					array(
						'name'  => 'expirationDate',
						'value' => 'XXXX',
					)
				),
			),
			array(
				'name' => 'solution',
				array(
					'name'  => 'id',
					'value' => 'yes' === $settings['testmode'] ? self::SANDBOX_SOLUTION_ID : self::PRODUCTION_SOLUTION_ID,
				),
			),
			array(
				'name'  => 'refTransId',
				'value' => wc_authorize_net_dpm_order_prop( $order, 'transaction_id' ),
			),
			array(
				'name' => 'order',
				array(
					'name'  => 'invoiceNumber',
					'value' => $order->get_order_number(),
				),
			),
		);

		return self::_request( 'createTransactionRequest', $request );
	}

	/**
	 * Cancel translation in given order_id that is not settled yet.
	 *
	 * @since 1.7.0
	 *
	 * @param int|WC_Order $order Order ID or order object.
	 *
	 * @return WP_Error|array See self::_request return value.
	 */
	public static function void( $order ) {
		$order          = wc_get_order( $order );
		$transaction_id = wc_authorize_net_dpm_order_prop( $order, 'transaction_id' );

		if ( empty( $transaction_id ) ) {
			return new WP_Error( 'missing_transaction_id', __( 'Transaction ID is missing from the order', 'woocommerce-gateway-authorize-net-dpm' ) );
		}

		$settings = wc_authorize_net_dpm()->get_settings();

		$request = array(
			'name' => 'transactionRequest',
			array(
				'name'  => 'transactionType',
				'value' => 'voidTransaction',
			),
			array(
				'name' => 'solution',
				array(
					'name'  => 'id',
					'value' => 'yes' === $settings['testmode'] ? self::SANDBOX_SOLUTION_ID : self::PRODUCTION_SOLUTION_ID,
				),
			),
			array(
				'name'  => 'refTransId',
				'value' => $transaction_id,
			),
		);

		return self::_request( 'createTransactionRequest', $request );
	}

	/**
	 * Get transaction details of a given order_id.
	 *
	 * @since 1.7.0
	 *
	 * @param int|WC_Order $order Order ID or order object.
	 *
	 * @return WP_Error|array See self::_request return value
	 */
	public static function get_transaction_details( $order ) {
		$order = wc_get_order( $order );
		$transaction_id = wc_authorize_net_dpm_order_prop( $order, 'transaction_id' );

		if ( empty( $transaction_id ) ) {
			return new WP_Error( 'missing_transaction_id', __( 'Transaction ID is missing from the order', 'woocommerce-gateway-authorize-net-dpm' ) );
		}

		$request = array(
			'name'  => 'transId',
			'value' => $transaction_id,
		);

		return self::_request( 'getTransactionDetailsRequest', $request );
	}
}
