<?php
/**
 * Authorize.Net DPM Form.
 *
 * @package WC_Authorize_Net_DPM
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Authorize.Net DPM Form.
 *
 * @since 1.7.0
 */
class WC_Authorize_Net_Form {

	const PRODUCTION_URL = 'https://secure2.authorize.net/gateway/transact.dll';
	const SANDBOX_URL = 'https://test.authorize.net/gateway/transact.dll';

	/**
	 * Fields to render in Direct Post Method.
	 *
	 * @since 1.7.0
	 *
	 * @var array
	 */
	protected static $_fields = array(
		'x_address',
		'x_amount',
		'x_background_url',
		'x_card_num',
		'x_city',
		'x_color_background',
		'x_color_link',
		'x_color_text',
		'x_company',
		'x_country',
		'x_cust_id',
		'x_customer_ip',
		'x_description',
		'x_delim_data',
		'x_duplicate_window',
		'x_duty',
		'x_email',
		'x_email_customer',
		'x_fax',
		'x_first_name',
		'x_footer_email_receipt',
		'x_footer_html_payment_form',
		'x_footer_html_receipt',
		'x_fp_hash',
		'x_fp_sequence',
		'x_fp_timestamp',
		'x_freight',
		'x_header_email_receipt',
		'x_header_html_payment_form',
		'x_header_html_receipt',
		'x_invoice_num',
		'x_last_name',
		'x_line_item',
		'x_login',
		'x_logo_url',
		'x_method',
		'x_phone',
		'x_po_num',
		'x_receipt_link_method',
		'x_receipt_link_text',
		'x_receipt_link_url',
		'x_recurring_billing',
		'x_relay_response',
		'x_relay_url',
		'x_rename',
		'x_ship_to_address',
		'x_ship_to_company',
		'x_ship_to_country',
		'x_ship_to_city',
		'x_ship_to_first_name',
		'x_ship_to_last_name',
		'x_ship_to_state',
		'x_ship_to_zip',
		'x_show_form',
		'x_state',
		'x_tax',
		'x_tax_exempt',
		'x_test_request',
		'x_trans_id',
		'x_type',
		'x_version',
		'x_zip',
	);

	/**
	 * Get CC form for Direct Post Method.
	 *
	 * @param int|WC_Order $order Order ID or order object.
	 *
	 * @return string HTML form
	 */
	public static function get_cc_form( $order ) {
		$settings = wc_authorize_net_dpm()->get_settings();
		$order    = wc_get_order( $order );
		$time     = time();
		$fp       = self::get_fingerprint( wc_authorize_net_dpm_order_prop( $order, 'id' ), $time, $order->get_total() );

		$values = apply_filters( 'wc_authorize_net_dpm_request_data', array(
			'x_amount'             => $order->get_total(),
			'x_fp_sequence'        => wc_authorize_net_dpm_order_prop( $order, 'id' ),
			'x_fp_hash'            => $fp,
			'x_fp_timestamp'       => $time,
			'x_relay_response'     => 'TRUE',
			'x_relay_url'          => WC_Authorize_Net_Relay_Response_Handler::get_url(),
			'x_login'              => $settings['api_login'],

			'x_first_name'         => wc_authorize_net_dpm_order_prop( $order, 'billing_first_name' ),
			'x_last_name'          => wc_authorize_net_dpm_order_prop( $order, 'billing_last_name' ),
			'x_company'            => wc_authorize_net_dpm_order_prop( $order, 'billing_company' ),
			'x_address'            => wc_authorize_net_dpm_order_prop( $order, 'billing_address_1' ) . ' ' . wc_authorize_net_dpm_order_prop( $order, 'billing_address_2' ),
			'x_city'               => wc_authorize_net_dpm_order_prop( $order, 'billing_city' ),
			'x_state'              => wc_authorize_net_dpm_order_prop( $order, 'billing_state' ),
			'x_zip'                => wc_authorize_net_dpm_order_prop( $order, 'billing_postcode' ),
			'x_country'            => wc_authorize_net_dpm_order_prop( $order, 'billing_country' ),
			'x_phone'              => wc_authorize_net_dpm_order_prop( $order, 'billing_phone' ),
			'x_email'              => wc_authorize_net_dpm_order_prop( $order, 'billing_email' ),

			'x_ship_to_first_name' => wc_authorize_net_dpm_order_prop( $order, 'shipping_first_name' ),
			'x_ship_to_last_name'  => wc_authorize_net_dpm_order_prop( $order, 'shipping_last_name' ),
			'x_ship_to_company'    => wc_authorize_net_dpm_order_prop( $order, 'shipping_company' ),
			'x_ship_to_address'    => wc_authorize_net_dpm_order_prop( $order, 'shipping_address_1' ) . ' ' . wc_authorize_net_dpm_order_prop( $order, 'shipping_address_2' ),
			'x_ship_to_city'       => wc_authorize_net_dpm_order_prop( $order, 'shipping_city' ),
			'x_ship_to_state'      => wc_authorize_net_dpm_order_prop( $order, 'shipping_state' ),
			'x_ship_to_zip'        => wc_authorize_net_dpm_order_prop( $order, 'shipping_postcode' ),
			'x_ship_to_country'    => wc_authorize_net_dpm_order_prop( $order, 'shipping_country' ),

			'x_cust_id'            => wc_authorize_net_dpm_order_prop( $order, 'user_id' ),
			'x_invoice_num'        => $order->get_order_number(),
			'x_order_id'           => wc_authorize_net_dpm_order_prop( $order, 'id' ),
			'x_order_key'          => wc_authorize_net_dpm_order_prop( $order, 'order_key' ),
			'x_description'        => sprintf( __( 'Order %s', 'woocommerce-gateway-authorize-net-dpm' ), $order->get_order_number() ),
			'x_type'               => 'yes' === $settings['auth_only'] ? 'AUTH_ONLY' : 'AUTH_CAPTURE',

			'x_solution_id'        => 'yes' === $settings['testmode'] ? WC_Authorize_Net_API::SANDBOX_SOLUTION_ID : WC_Authorize_Net_API::PRODUCTION_SOLUTION_ID,
		) );

		$hidden_fields = self::get_html_fields( $order, $values );

		ob_start();

		echo '<form method="post" action="' . esc_attr( self::get_post_form_url() ) . '"> ' . $hidden_fields;

		wc_get_template( 'cc-form.php', array(
			'prefill' => 'yes' === $settings['testmode'],
		), 'authorize-net-dpm/', plugin_dir_path( dirname( __FILE__ ) ) . 'templates/' );

		echo '</form>';

		return ob_get_clean();
	}

	/**
	 * Get post form URL.
	 *
	 * @since 1.7.0
	 *
	 * @return string Post form URL
	 */
	public static function get_post_form_url() {
		$settings = wc_authorize_net_dpm()->get_settings();
		return 'yes' === $settings['testmode'] ? self::SANDBOX_URL : self::PRODUCTION_URL;
	}

	/**
	 * Get unique transaction fingerprint.
	 *
	 * @since 1.7.0
	 *
	 * @see http://www.authorize.net/content/dam/authorize/documents/SIM_guide.pdf
	 *
	 * @param string $fp_sequence  An invoice number or random number.
	 * @param string $fp_timestamp Timestamp at the time of fingerprint generation.
	 * @param string $amount       Amount of transaction.
	 *
	 * @return string Fingerprint
	 */
	public static function get_fingerprint( $fp_sequence, $fp_timestamp, $amount ) {
		$settings = wc_authorize_net_dpm()->get_settings();

		$fp_input = sprintf( '%s^%s^%s^%s^', $settings['api_login'], $fp_sequence, $fp_timestamp, $amount );

		if ( function_exists( 'hash_hmac' ) ) {
			return hash_hmac( 'md5', $fp_input, $settings['transaction_key'] );
		}

		return bin2hex( mhash( MHASH_MD5, $fp_input, $settings['transaction_key'] ) );
	}

	/**
	 * Get HTML fields to render.
	 *
	 * @since 1.7.0
	 *
	 * @param int|WC_Order $order  Order ID or order object.
	 * @param array        $values Values to set in fields.
	 *
	 * @return string HTML of fields
	 */
	public static function get_html_fields( $order, $values ) {
		$order = wc_get_order( $order );

		$values = array_merge(
			array(
				'x_relay_response' => 'TRUE',
				'x_version'        => '3.1',
				'x_delim_char'     => ',',
				'x_delim_data'     => 'TRUE',
			),
			$values
		);

		$fields = array_fill_keys( self::$_fields, '' );
		$fields = array_merge( $fields, $values );

		wc_authorize_net_dpm()->log( sprintf( 'Data to POST to %s: %s', self::get_post_form_url(), print_r( $fields, true ) ) );

		$html        = '';
		$logged_data = array();
		foreach ( $fields as $field => $value ) {
			if ( $value ) {
				$html .= sprintf( '<input type="hidden" name="%s" value="%s">', esc_attr( $field ), esc_attr( $value ) );

				$logged_data[ $field ] = esc_attr( $value );
			}
		}

		$line_items = self::get_line_items( $order );
		if ( count( $line_items ) > 30 ) {
			$line_items = array_slice( $line_items, 0, 30 );
		}

		// x_line_item is special because merchant can submit as many as 30
		// distinct line items containing itemized order information per transaction.
		foreach ( $line_items as $i => $line_item ) {
			$html .= sprintf( '<input type="hidden" name="x_line_item" value="%s">', esc_attr( $line_item ) );

			$logged_data[ 'x_line_item (' . $i . ')' ] = esc_attr( $line_item );
		}

		return $html;
	}

	/**
	 * Get line items from order to be sent via DPM.
	 *
	 * @since 1.7.0
	 *
	 * @param WC_Order|int $order Order object or order ID.
	 *
	 * @return array Line items for x_line_item
	 */
	public static function get_line_items( $order ) {
		$line_items  = array();
		$order_items = $order->get_items( array( 'line_item', 'fee' ) );

		foreach ( $order_items as $item ) {
			if ( 'fee' !== $item['type'] ) {
				$product = $order->get_product_from_item( $item );
				if ( ! is_a( $product, 'WC_Product' ) ) {
					continue;
				}

				$item_name       = ! empty( $item['name'] ) ? substr( $item['name'], 0, 31 ) : '';
				$qty             = absint( $item['qty'] );
				$item_line_total = number_format( $order->get_item_subtotal( $item, false ), 2, '.', '' );

				$line_item = array(
					$product->get_id(),
					$item_name,
					'', // Item description, skip it.
					$qty,
					$item_line_total,
					$product->is_taxable() ? 'Y' : 'N',
				);

				$line_items[] = implode( '<|>', $line_item );
			}
		}

		return $line_items;
	}
}
