<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Authorize.Net relay response handler
 *
 * @since 1.7.0
 */
class WC_Authorize_Net_Relay_Response_Handler {

	const APPROVED = 1;
	const DECLINED = 2;
	const ERROR = 3;
	const HELD = 4;

	/**
	 * Get relay response URL.
	 *
	 * URL that set in x_relay_url for DPM. Authorize.Net will respond to this
	 * URL.
	 *
	 * @since 1.7.0
	 *
	 * @return string Relay response URL
	 */
	public static function get_url() {
		return WC()->api_request_url( 'WC_Gateway_Authorize_DPM' );
	}

	/**
	 * Checks whether a request is a valid request from Authorize.Net.
	 *
	 * @since 1.7.0
	 *
	 * @param array $data Posted data
	 *
	 * @return bool Returns true if request is valid
	 */
	public static function is_valid_request( $data ) {
		if ( empty( $data ) ) {
			wc_authorize_net_dpm()->log( sprintf( '%s - Posted data is empty', __METHOD__ ) );
			return false;
		}

		return self::_contains_valid_hash( $data );
	}

	/**
	 * Check whether posted data contains valid hash.
	 *
	 * @since 1.7.0
	 *
	 * @param array $data Posted data
	 *
	 * @return bool Returns true is data contains valid hash
	 */
	protected static function _contains_valid_hash( $data ) {
		$required_fields = array( 'x_SHA2_Hash' );

		foreach ( $required_fields as $field ) {
			if ( ! isset( $data[ $field ] ) ) {
				wc_authorize_net_dpm()->log( sprintf( '%s - Field %s is missing in posted data', __METHOD__, $field ) );
				return false;
			}
		}

		// Get plugin settings.
		$settings      = wc_authorize_net_dpm()->get_settings();
		$signature_key = $settings['signature_key'];

		// If no signature key has been set then skip the hash check (if a merchant hasn't set this yet we don't want to
		// prevent orders processing).
		if ( '' === $signature_key ) {
			wc_authorize_net_dpm()->log( 'No signature key set, hash check skipped. Please add your signature key in the plugin settings.' );
			return true;
		}

		// Hash the posted data.
		$expected_hash = self::_build_hash( $data, $signature_key );

		// Compare our generated hash against the posted hash.
		if ( $data['x_SHA2_Hash'] !== $expected_hash ) {
			wc_authorize_net_dpm()->log( sprintf( '%s - Expects x_SHA2_Hash to be %s, posted x_SHA2_Hash is %s', __METHOD__, $expected_hash, $data['x_SHA2_Hash'] ) );
			return false;
		}

		return true;
	}

	/**
	 * Build hash from posted data and the signature key.
	 *
	 * @param array  $data          Posted data.
	 * @param string $signature_key Merchant's signature key.
	 *
	 * @return string Built hash
	 */
	private static function _build_hash( $data, $signature_key ) {
		// The fields to hash.
		$hash_fields = array(
			'x_trans_id',
			'x_test_request',
			'x_response_code',
			'x_auth_code',
			'x_cvv2_resp_code',
			'x_cavv_response',
			'x_avs_code',
			'x_method',
			'x_account_number',
			'x_amount',
			'x_company',
			'x_first_name',
			'x_last_name',
			'x_address',
			'x_city',
			'x_state',
			'x_zip',
			'x_country',
			'x_phone',
			'x_fax',
			'x_email',
			'x_ship_to_company',
			'x_ship_to_first_name',
			'x_ship_to_last_name',
			'x_ship_to_address',
			'x_ship_to_city',
			'x_ship_to_state',
			'x_ship_to_zip',
			'x_ship_to_country',
			'x_invoice_num',
		);

		// Combine the fields to hash in a list delimited with carets.
		$text_to_hash = '^';
		foreach ( $hash_fields as $hash_field ) {
			$value = ! empty( $data[ $hash_field ] ) ? $data[ $hash_field ] : '';

			$text_to_hash = $text_to_hash . $value . '^';
		}

		// Hash the string with the signature string.
		if ( function_exists( 'hash_hmac' ) ) {
			$expected_hash = hash_hmac( 'sha512', $text_to_hash, hex2bin( $signature_key ) );
		} else {
			$expected_hash = bin2hex( mhash( MHASH_SHA512, $text_to_hash, hex2bin( $signature_key ) ) );
		}

		// Authorize.Net provides the hash uppercase, so do the same to ours.
		$expected_hash = strtoupper( $expected_hash );

		return $expected_hash;
	}

	/**
	 * Checks whether posted data tells the transaction is approved.
	 *
	 * @since 1.7.0
	 *
	 * @param array $data Posted data
	 *
	 * @return bool Returns true is transaction is approved
	 */
	public static function is_approved( $data ) {
		if ( empty( $data['x_response_code'] ) ) {
			return false;
		}

		return intval( $data['x_response_code'] ) === self::APPROVED;
	}

	/**
	 * Checks whether posted data tells the transaction is declined.
	 *
	 * @since 1.7.0
	 *
	 * @param array $data Posted data
	 *
	 * @return bool Returns true is transaction is declined
	 */
	public static function is_declined( $data ) {
		if ( empty( $data['x_response_code'] ) ) {
			return false;
		}

		return intval( $data['x_response_code'] ) === self::DECLINED;
	}

	/**
	 * Checks whether posted data tells the transaction results in error.
	 *
	 * @since 1.7.0
	 *
	 * @param array $data Posted data
	 *
	 * @return bool Returns true is transaction results in error
	 */
	public static function is_error( $data ) {
		if ( empty( $data['x_response_code'] ) ) {
			return false;
		}

		return intval( $data['x_response_code'] ) === self::ERROR;
	}

	/**
	 * Checks whether posted data tells the transaction is held.
	 *
	 * @since 1.7.0
	 *
	 * @param array $data Posted data
	 *
	 * @return bool Returns true is transaction is held
	 */
	public static function is_held( $data ) {
		if ( empty( $data['x_response_code'] ) ) {
			return false;
		}

		return intval( $data['x_response_code'] ) === self::HELD;
	}

	/**
	 * This will be sent to Authorize.Net, after veryfing POST'ed data to self::get_url(),
	 * to redirect buyer back to merchant's server.
	 *
	 * @since 1.7.0
	 *
	 * @param string $redirect_url Redirect URL
	 *
	 * @return string Temporary redirect page to redirect 
	 */
	public static function get_redirect_page_html( $redirect_url ) {
		return "<html><head><script language=\"javascript\">
			<!--
			window.location=\"{$redirect_url}\";
			//-->
			</script>
			</head><body><noscript><meta http-equiv=\"refresh\" content=\"1;url={$redirect_url}\"></noscript></body></html>";
	}
}
