<?php
/**
 * Authorize.net DPM gateway class
 *
 * @package WC_Authorize_Net_DPM
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Authorize.net DPM gateway class
 */
class WC_Gateway_Authorize_DPM extends WC_Payment_Gateway {

	/**
	 * Constructor
	 */
	public function __construct() {
		$this->id                 = 'authorize_net_dpm';
		$this->method_title       = __( 'Authorize.net DPM', 'woocommerce-gateway-authorize-net-dpm' );
		$this->method_description = __( 'Authorize.net DPM (direct post method) handles all the steps in the secure transaction while remaining virtually transparent. Payment data is passed from the checkout to Authorize.net for processing thus removing the necessity of SSL and simplifying PCI compliance.', 'woocommerce-gateway-authorize-net-dpm' );
		$this->icon               = apply_filters( 'wc_authorize_net_dpm_icon', plugins_url( '/assets/images/cards.png', dirname( __FILE__ ) ) );

		$this->supports = array(
			'products',
			'refunds',
		);

		// Load the form fields.
		$this->init_form_fields();

		// Load the settings.
		$this->init_settings();

		// Load settings.
		$this->load_settings();

		// Hooks.
		add_action( 'woocommerce_receipt_authorize_net_dpm', array( $this, 'receipt_page' ) );
		add_action( 'woocommerce_api_wc_gateway_authorize_dpm', array( $this, 'relay_response' ) );
		add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( $this, 'process_admin_options' ) );
		add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( $this, 'validate_api_keys' ) );
	}

	/**
	 * Initialise Gateway Settings Form Fields.
	 */
	public function init_form_fields() {
		$this->form_fields = include( 'gateway-settings.php' );
	}

	/**
	 * Load saved settings into gateway instance's properties.
	 *
	 * @since 1.7.0
	 */
	public function load_settings() {
		$this->title           = $this->get_option( 'title' );
		$this->description     = $this->get_option( 'description' );
		$this->enabled         = $this->get_option( 'enabled' );
		$this->api_login       = $this->get_option( 'api_login' );
		$this->transaction_key = $this->get_option( 'transaction_key' );
		$this->signature_key   = $this->get_option( 'signature_key' );
		$this->auth_only       = 'yes' === $this->get_option( 'auth_only' );
		$this->testmode        = 'yes' === $this->get_option( 'testmode' );
	}

	/**
	 * Validate API keys when settings are updated.
	 *
	 * @since 1.7.0
	 */
	public function validate_api_keys() {
		$this->load_settings();

		try {
			if ( empty( $this->api_login ) || empty( $this->transaction_key ) ) {
				throw new Exception( __( 'API Login ID or Transaction Key cannot be left empty.', 'woocommerce-gateway-authorize-net-dpm' ) );
			}

			if ( empty( $this->signature_key ) ) {
				throw new Exception( __( 'Signature Key cannot be left empty.', 'woocommerce-gateway-authorize-net-dpm' ) );
			}

			$resp = WC_Authorize_Net_API::authenticate();
			if ( is_wp_error( $resp ) ) {
				throw new EXception( __( 'User authentication failed. Make sure entered API Login ID and Transaction Key are correct or please try again.', 'woocommerce-gateway-authorize-net-dpm' ) );
			}
		} catch ( Exception $e ) {
			WC_Admin_Settings::add_error( $e->getMessage() );
		}
	}

	/**
	 * Check if this gateway is enabled
	 */
	public function is_available() {
		if ( ! $this->api_login || ! $this->transaction_key ) {
			return false;
		}
		return parent::is_available();
	}

	/**
	 * Process the payment and return the result - this will redirect the customer
	 * to the pay page.
	 *
	 * @param int $order_id Order ID.
	 *
	 * @return array Process payment response.
	 */
	public function process_payment( $order_id ) {
		$order = wc_get_order( $order_id );

		wc_authorize_net_dpm()->log( 'Processing order ' . $order_id );

		return array(
			'result'   => 'success',
			'redirect' => $order->get_checkout_payment_url( true ),
		);
	}

	/**
	 * Receipt_page for showing the payment form which sends data to authorize.net
	 *
	 * @param int $order_id Order ID.
	 */
	public function receipt_page( $order_id ) {
		wc_authorize_net_dpm()->log( 'Showing receipt page for order ' . $order_id );

		echo WC_Authorize_Net_Form::get_cc_form( $order_id );
	}

	/**
	 * Relay response - handles return data from Authorize.net and does redirects
	 */
	public function relay_response() {
		$data = ! empty( $_POST ) ? $_POST : array();
		if ( empty( $data ) ) {
			wc_authorize_net_dpm()->log( 'Empty POST data' );
			wp_die( __( 'No data to process.', 'woocommerce-gateway-authorize-net-dpm' ) );
		}

		wc_authorize_net_dpm()->log( sprintf( '%s - Receive request in %s with data: %s', __METHOD__, wc_clean( $_SERVER['REQUEST_URI'] ), print_r( $data, true ) ) );

		$order_id = ! empty( $data['x_order_id'] ) ? absint( $data['x_order_id'] ) : null;
		if ( ! $order_id ) {
			wc_authorize_net_dpm()->log( 'Invalid x_order_id in POST data' );
			wp_die( __( 'Missing order ID in submitted data.', 'woocommerce-gateway-authorize-net-dpm' ) );
		}

		if ( ! WC_Authorize_Net_Relay_Response_Handler::is_valid_request( $data ) ) {
			wc_authorize_net_dpm()->log( sprintf( '%s - Invalid POST data', __METHOD__ ) );

			wp_die( __( 'Invalid Authorize.Net data. Check your Signature Key setting.', 'woocommerce-gateway-authorize-net-dpm' ) );
		}

		$order = wc_get_order( $order_id );
		if ( ! $order ) {
			wc_authorize_net_dpm()->log( sprintf( '%s - Unknown order ID %s', __METHOD__, $order_id ) );
			wp_die( __( 'Invalid order.', 'woocommerce-gateway-authorize-net-dpm' ) );
		}

		// POST'ed data is valid and order exists.
		status_header( 200 );

		wc_authorize_net_dpm()->log( sprintf( '%s - Processing data from Authorize.Net', __METHOD__ ) );

		$transaction_id = ! empty( $data['x_trans_id'] ) ? $data['x_trans_id'] : '';
		$order_key      = ! empty( $data['x_order_key'] ) ? $data['x_order_key'] : '';
		$amount         = ! empty( $data['x_amount'] ) ? $data['x_amount'] : 0;
		$code           = ! empty( $data['x_response_code'] ) ? $data['x_response_code'] : '';
		$reason         = ! empty( $data['x_response_reason_text'] ) ? $data['x_response_reason_text'] : '';
		$reason_code    = ! empty( $data['x_response_reason_code'] ) ? $data['x_response_reason_code'] : '';
		$redirect_url   = $this->get_return_url( $order );

		try {
			if ( WC_Authorize_Net_Relay_Response_Handler::is_approved( $data ) ) {
				if ( empty( $transaction_id ) && ! $this->testmode ) {
					throw new Exception( __( 'Payment failure: Invalid transaction ID.', 'woocommerce-gateway-authorize-net-dpm' ) );
				}

				if ( ! $order->key_is_valid( $order_key ) ) {
					if ( $order->has_status( array( 'pending', 'failed' ) ) ) {
						$order->update_status( 'on-hold', sprintf( __( 'Payment received, but order ID did not match key: code %1$s - %2$s.', 'woocommerce-gateway-authorize-net-dpm' ), $code, $reason ) );
						if ( version_compare( WC_VERSION, '3.0', '<' ) ) {
							$order->reduce_order_stock();
						} else {
							wc_reduce_stock_levels( $order->get_id() );
						}
					}
				} elseif ( $order->get_total() != $amount ) {
					if ( $order->has_status( array( 'pending', 'failed' ) ) ) {
						$order->update_status( 'on-hold', sprintf( __( 'Payment received, but order total did not match %1$s != %2$s.', 'woocommerce-gateway-authorize-net-dpm' ), $amount, $order->get_total() ) );
						if ( version_compare( WC_VERSION, '3.0', '<' ) ) {
							$order->reduce_order_stock();
						} else {
							wc_reduce_stock_levels( $order->get_id() );
						}
					}
				} else {

					if ( $this->auth_only ) {
						update_post_meta( wc_authorize_net_dpm_order_prop( $order, 'id' ), '_transaction_id', $transaction_id );
						$order->update_status( 'on-hold', __( 'Authorize.Net transaction is authorized. Process the order to take payment, or cancel to void the transaction.', 'woocommerce-gateway-authorize-net-dpm' ) );
					} else {
						$order->payment_complete( $transaction_id );
					}

					$redirect_url = add_query_arg( array( 'response_code' => WC_Authorize_Net_Relay_Response_Handler::APPROVED, 'transaction_id' => $transaction_id ), $redirect_url );
				}
			} elseif ( WC_Authorize_Net_Relay_Response_Handler::is_held( $data ) ) {

				$order->update_status( 'on-hold', __( 'Authorize.net payment on-hold.', 'woocommerce-gateway-authorize-net-dpm' ) );
				if ( version_compare( WC_VERSION, '3.0', '<' ) ) {
					$order->reduce_order_stock();
				} else {
					wc_reduce_stock_levels( $order->get_id() );
				}
				$redirect_url = add_query_arg( array( 'response_code' => WC_Authorize_Net_Relay_Response_Handler::HELD, 'transaction_id' => $transaction_id ), $redirect_url );

			} else {
				throw new Exception( sprintf( __( 'Payment failure: code %1$s - %2$s (%3$s).', 'woocommerce-gateway-authorize-net-dpm' ), $code, $reason, $reason_code ) );
			}

			WC()->cart->empty_cart();

		} catch ( Exception $e ) {
			$order->update_status( 'failed', $e->getMessage() );
			$redirect_url = add_query_arg( 'wc_error', $e->getMessage(), $order->get_checkout_payment_url( true ) );
		}

		wc_authorize_net_dpm()->log( sprintf( '%s - Outputting redirect to %s', __METHOD__, esc_attr( $redirect_url ) ) );

		echo WC_Authorize_Net_Relay_Response_Handler::get_redirect_page_html( $redirect_url );
		exit;
	}

	/**
	 * Refund the transaction.
	 *
	 * @since 1.7.0
	 *
	 * @param int    $order_id Order ID.
	 * @param float  $amount   Amount to refund.
	 * @param string $reason   Reason to refund.
	 *
	 * @return bool|WP_Error Returns bool or WP_Error.
	 */
	public function process_refund( $order_id, $amount = null, $reason = '' ) {
		$order = wc_get_order( $order_id );

		if ( ! $order ) {
			return false;
		}

		wc_authorize_net_dpm()->log( sprintf( '%s - Refund order #%s with the amount of %s', __METHOD__, $order_id, $amount ) );

		$resp = WC_Authorize_Net_API::refund( $order, $amount, $reason );
		if ( is_wp_error( $resp ) ) {
			wc_authorize_net_dpm()->log( sprintf( '%s - Error: %s', __METHOD__, $resp->get_error_message() ) );
			return $resp;
		}

		$message = sprintf( __( 'Successfully refunded Authorize.Net transaction %1$s with amount %2$s', 'woocommerce-gateway-authorize-net-dpm' ), $order->get_transaction_id(), wc_price( $amount ) );

		$order->add_order_note( $message );

		wc_authorize_net_dpm()->log( sprintf( '%s - Refund order #%s succeed', __METHOD__, $order_id ) );

		return true;
	}
}
