<?php
/**
 * Plugin Name: WooCommerce Authorize.net DPM Gateway
 * Plugin URI: https://woocommerce.com/products/authorize-net-dpm/
 * Description: Extends WooCommerce with an <a href="https://www.authorize.net" target="_blank">Authorize.net</a> DPM (Direct Post Method) gateway. An Authorize.net merchant account is required.
 * Version: 1.8.0
 * Author: WooCommerce
 * Author URI: https://woocommerce.com/
 * Text Domain: woocommerce-gateway-authorize-net-dpm
 * Domain Path: /languages/
 * Tested up to: 5.0
 * WC tested up to: 3.6
 * WC requires at least: 2.6
 * Woo: 18654:9a55f12d866225c031a800cb21b22c78
 *
 * Copyright: © 2009-2017 WooCommerce.
 * License: GNU General Public License v3.0
 * License URI: http://www.gnu.org/licenses/gpl-3.0.html
 * DPM Docs: http://www.authorize.net/support/SIM_guide.pdf
 *
 * @package WC_Authorize_Net_DPM
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Required functions
 */
if ( ! function_exists( 'woothemes_queue_update' ) ) {
	require_once( 'woo-includes/woo-functions.php' );
}

/**
 * Plugin updates.
 */
woothemes_queue_update( plugin_basename( __FILE__ ), '9a55f12d866225c031a800cb21b22c78', '18654' );


/**
 * WC Authorize Net DPM main class.
 *
 * @since 1.7.0
 */
class WC_Authorize_Net_DPM {

	/**
	 * Plugin's main file.
	 *
	 * @since 1.7.0
	 *
	 * @var string
	 */
	public $main_file;

	/**
	 * Plugin dir, without trailing slash.
	 *
	 * @since 1.7.0
	 *
	 * @var string
	 */
	public $plugin_dir;

	/**
	 * Cached gateway settings so we don't need to call get_option multiple
	 * times.
	 *
	 * @since 1.7.0
	 *
	 * @var array
	 */
	private $_settings = array();

	/**
	 * Order handler.
	 *
	 * @since 1.7.0
	 *
	 * @var WC_Authorize_Net_Order_Handler
	 */
	private $_order_handler;

	/**
	 * Relay response handler.
	 *
	 * @since 1.7.0
	 *
	 * @var WC_Authorize_Net_Relay_Response_Handler
	 */
	private $_relay_response_handler;

	/**
	 * Logger instance.
	 *
	 * @since 1.7.0
	 *
	 * @var WC_Logger
	 */
	private $_logger;

	/**
	 * Constructor.
	 *
	 * @since 1.7.0
	 */
	public function __construct() {
		$this->main_file  = __FILE__;
		$this->plugin_dir = untrailingslashit( plugin_dir_path( __FILE__ ) );

		$this->load_includes();

		// Show notices for any missing settings.
		add_filter( 'admin_notices', array( $this, 'show_missing_setting_notice' ) );

		add_action( 'plugins_loaded', array( $this, 'plugins_loaded' ), 0 );
		add_filter( 'plugin_action_links_' . plugin_basename( __FILE__ ), array( $this, 'plugin_action_links' ) );
	}

	/**
	 * Load other classes.
	 *
	 * @since 1.7.0
	 */
	public function load_includes() {
		require_once( $this->plugin_dir . '/includes/class-wc-authorize-net-api.php' );
		require_once( $this->plugin_dir . '/includes/class-wc-authorize-net-order-handler.php' );
		require_once( $this->plugin_dir . '/includes/class-wc-authorize-net-relay-response-handler.php' );
		require_once( $this->plugin_dir . '/includes/class-wc-authorize-net-form.php' );
	}

	/**
	 * Handler when plugins loaded.
	 *
	 * Do the followings:
	 *
	 * - Load and init our gateway
	 * - Load other handlers
	 * - Load translated strings
	 *
	 * @since 1.7.0
	 */
	public function plugins_loaded() {
		if ( ! class_exists( 'WC_Payment_Gateway' ) ) {
			return;
		}

		require_once( $this->plugin_dir . '/includes/class-wc-authorize-net-privacy.php' );

		// Load gateway and register gateway.
		require_once( $this->plugin_dir . '/includes/class-wc-gateway-authorize-dpm.php' );
		add_filter( 'woocommerce_payment_gateways', array( $this, 'register_gateway' ) );

		// Load other handlers.
		$this->_order_handler          = new WC_Authorize_Net_Order_Handler();
		$this->_relay_response_handler = new WC_Authorize_Net_Relay_Response_Handler();

		// Load translated strings.
		load_plugin_textdomain( 'woocommerce-gateway-authorize-net-dpm', false, dirname( plugin_basename( __FILE__ ) ) . '/languages' );
	}

	/**
	 * Register gateway.
	 *
	 * @since 1.7.0
	 *
	 * @param array $gateways Gateways.
	 *
	 * @return array Gateways
	 */
	public function register_gateway( $gateways ) {
		$gateways[] = 'WC_Gateway_Authorize_DPM';
		return $gateways;
	}

	/**
	 * Show admin notices for any missing settings.
	 */
	public function show_missing_setting_notice() {
		// Only show a missing setting notice if the gateway is active.
		$settings = $this->get_settings();
		if ( 'yes' !== $settings['enabled'] ) {
			return;
		}

		// If a signature key has been set then don't show the notice.
		if ( '' !== $settings['signature_key'] ) {
			return;
		}

		// Build the message to show in the notice.
		$message = sprintf(

			/*
			 * translators:
			 * %1: Hash changes documentation URL
			 * %2: What is a signature key documentation URL
			 * %3: Plugin settings page URL
			 */
			__(
				'Some <a href="%1$s">changes</a> have been made to how Authorize.Net DPM secures payments to your store. Please generate a signature key for your Authorize.Net account by following the instructions found <a href="%2$s">here</a>. Once created it needs to be added to the Authorize.Net DPM plugin <a href="%3$s">settings</a>.',
				'woocommerce-gateway-authorize-net-dpm'
			),
			'https://support.authorize.net/s/article/MD5-Hash-End-of-Life-Signature-Key-Replacement',
			'https://support.authorize.net/s/article/What-is-a-Signature-Key',
			$this->get_settings_url()
		);

		// Output the notice.
		?>
		<div class="notice notice-error">
			<p><?php echo $message; ?></p>
		</div>
		<?php
	}

	/**
	 * Add custom action links on the plugin screen.
	 *
	 * @since 1.7.0
	 *
	 * @param  mixed $actions Plugin Actions Links.
	 * @return array          Action links
	 */
	public function plugin_action_links( $actions ) {
		$use_id_as_section = version_compare( WC()->version, '2.6', '>=' );

		$section_slug = $use_id_as_section ? 'authorize_net_dpm' : strtolower( 'WC_Gateway_Authorize_DPM ' );
		$setting_link = admin_url( 'admin.php?page=wc-settings&tab=checkout&section=' . $section_slug );

		$custom_actions = array();

		$custom_actions['settings'] = sprintf( '<a href="%s">%s</a>', $setting_link, __( 'Settings', 'woocommerce-gateway-authorize-net-dpm' ) );
		$custom_actions['docs']     = sprintf( '<a href="%s">%s</a>', 'https://docs.woocommerce.com/document/authorize-net-dpm/', __( 'Docs', 'woocommerce-gateway-authorize-net-dpm' ) );

		// add the links to the front of the actions list.
		return array_merge( $custom_actions, $actions );
	}

	/**
	 * Wrapper to retrieve gateway settings.
	 *
	 * @since 1.7.0
	 *
	 * @return array Settings
	 */
	public function get_settings() {
		if ( ! empty( $this->_settings ) ) {
			return $this->_settings;
		}

		$defaults = array(
			'enabled'         => 'no',
			'title'           => '',
			'description'     => '',
			'api_login'       => '',
			'transaction_key' => '',
			'signature_key'   => '',
			'testmode'        => 'yes',
			'debug'           => 'yes',
			'auth_only'       => 'no',
		);

		$this->_settings = get_option( 'woocommerce_authorize_net_dpm_settings', $defaults );

		return $this->_settings;
	}

	/**
	 * Write a message to log if we're in "debug" mode.
	 *
	 * @since 1.7.0
	 *
	 * @param string $message Message to log.
	 */
	public function log( $message ) {
		$settings = $this->get_settings();

		if ( empty( $settings['debug'] ) ) {
			return;
		}

		if ( 'yes' !== $settings['debug'] ) {
			return;
		}

		if ( ! is_a( $this->_logger, 'WC_Logger' ) ) {
			$this->_logger = new WC_Logger();
		}

		$this->_logger->add( 'authorize_net_dpm', $message );

		if ( defined( 'WP_DEBUG' ) && WP_DEBUG ) {
			error_log( $message );
		}
	}

	/**
	 * Returns the URL of the configuration screen for this gateway, for use in internal links.
	 *
	 * @return string URL of the configuration screen for this gateway
	 */
	private function get_settings_url() {
		return admin_url( 'admin.php?page=wc-settings&tab=checkout&section=authorize_net_dpm' );
	}
}

/**
 * Get order property with compatibility check on order getter introduced
 * in WC 3.0.
 *
 * @since 1.7.2
 *
 * @param WC_Order $order Order object.
 * @param string   $prop  Property name.
 *
 * @return mixed Property value
 */
function wc_authorize_net_dpm_order_prop( $order, $prop ) {
	$getter = array( $order, 'get_' . $prop );
	return is_callable( $getter ) ? call_user_func( $getter ) : $order->{ $prop };
}

/**
 * Return instance of WC_Authorize_Net_DPM.
 *
 * @since 1.7.0
 *
 * @return WC_Authorize_Net_DPM
 */
function wc_authorize_net_dpm() {
	static $instance;

	if ( ! isset( $instance ) ) {
		$instance = new WC_Authorize_Net_DPM();
	}

	return $instance;
}
wc_authorize_net_dpm();
